"""Git repository analyzer."""

import subprocess
from datetime import datetime

from .models import CommitStats, FileStats, RangeStats


class GitAnalyzer:
    """Analyzes git repository statistics."""

    def __init__(self, repo_path: str = "."):
        """Initialize analyzer with repository path."""
        self.repo_path = repo_path

    def get_commit_stats(self, commit_hash: str = "HEAD") -> CommitStats:
        """Get statistics for a single commit."""
        # Get commit info
        cmd = [
            "git",
            "-C",
            self.repo_path,
            "show",
            "--format=%H|%an|%ad|%s",
            "--date=iso",
            "--numstat",
            commit_hash,
        ]
        result = subprocess.run(
            cmd, capture_output=True, text=True, check=True
        )

        lines = result.stdout.strip().split("\n")
        header = lines[0].split("|")

        commit_hash = header[0]
        author = header[1]
        date = datetime.fromisoformat(header[2].replace(" ", "T"))
        message = header[3]

        files = []
        total_added = 0
        total_deleted = 0

        for line in lines[2:]:  # Skip header and empty line
            if line.strip():
                parts = line.split("\t")
                if len(parts) >= 3:
                    added = int(parts[0]) if parts[0] != "-" else 0
                    deleted = int(parts[1]) if parts[1] != "-" else 0
                    path = parts[2]

                    files.append(
                        FileStats(
                            path=path,
                            lines_added=added,
                            lines_deleted=deleted,
                            lines_changed=added + deleted,
                        )
                    )

                    total_added += added
                    total_deleted += deleted

        return CommitStats(
            hash=commit_hash,
            author=author,
            date=date,
            message=message,
            files_changed=len(files),
            lines_added=total_added,
            lines_deleted=total_deleted,
            files=files,
        )

    def get_range_analytics(
        self, since: str, until: str = "HEAD"
    ) -> RangeStats:
        """Get analytics for a range of commits."""
        # Get commit list
        cmd = [
            "git",
            "-C",
            self.repo_path,
            "log",
            f"--since={since}",
            f"--until={until}",
            "--format=%H",
            "--reverse",
        ]
        result = subprocess.run(
            cmd, capture_output=True, text=True, check=True
        )

        commit_hashes = [
            line.strip()
            for line in result.stdout.strip().split("\n")
            if line.strip()
        ]

        commits = []
        authors = {}
        total_files = 0
        total_added = 0
        total_deleted = 0

        for commit_hash in commit_hashes:
            try:
                commit_stats = self.get_commit_stats(commit_hash)
                commits.append(commit_stats)

                # Update author stats
                authors[commit_stats.author] = (
                    authors.get(commit_stats.author, 0) + 1
                )

                # Update totals
                total_files += commit_stats.files_changed
                total_added += commit_stats.lines_added
                total_deleted += commit_stats.lines_deleted

            except subprocess.CalledProcessError:
                continue  # Skip problematic commits

        start_date = commits[0].date if commits else datetime.now()
        end_date = commits[-1].date if commits else datetime.now()

        return RangeStats(
            start_date=start_date,
            end_date=end_date,
            total_commits=len(commits),
            total_files_changed=total_files,
            total_lines_added=total_added,
            total_lines_deleted=total_deleted,
            commits=commits,
            authors=authors,
        )



"""Standard output formatter."""

import colorama
from colorama import Fore, Style

from ..core.models import CommitStats, RangeStats

# Initialize colorama for cross-platform color support
colorama.init()

class StandardFormatter:
    """Standard text formatter for Beacon delivery analytics output."""

    def format_commit_stats(self, stats: CommitStats) -> str:
        """Format commit statistics as standard text."""
        output = []
        output.append(f"{Fore.CYAN}Commit:{Style.RESET_ALL} {stats.hash[:8]}")
        output.append(f"{Fore.CYAN}Author:{Style.RESET_ALL} {stats.author}")
        output.append(f"{Fore.CYAN}Date:{Style.RESET_ALL} {stats.date.strftime('%Y-%m-%d %H:%M:%S')}")
        output.append(f"{Fore.CYAN}Message:{Style.RESET_ALL} {stats.message}")
        output.append("")
        output.append(f"{Fore.YELLOW}Files changed:{Style.RESET_ALL} {stats.files_changed}")
        output.append(f"{Fore.GREEN}Lines added:{Style.RESET_ALL} {stats.lines_added}")
        output.append(f"{Fore.RED}Lines deleted:{Style.RESET_ALL} {stats.lines_deleted}")
        
        net_change = stats.lines_added - stats.lines_deleted
        net_color = Fore.GREEN if net_change >= 0 else Fore.RED
        output.append(f"{Fore.YELLOW}Net change:{Style.RESET_ALL} {net_color}{net_change}{Style.RESET_ALL}")

        if stats.files:
            output.append("")
            output.append(f"{Fore.MAGENTA}File changes:{Style.RESET_ALL}")
            for file_stat in stats.files:
                output.append(
                    f"  {file_stat.path}: "
                    f"{Fore.GREEN}+{file_stat.lines_added}{Style.RESET_ALL} "
                    f"{Fore.RED}-{file_stat.lines_deleted}{Style.RESET_ALL}"
                )

        return "\n".join(output)

    def format_range_stats(self, stats: RangeStats) -> str:
        """Format range statistics as standard text."""
        output = []
        output.append(
            f"{Fore.CYAN}Range Analysis:{Style.RESET_ALL} {stats.start_date.strftime('%Y-%m-%d')} to "
            f"{stats.end_date.strftime('%Y-%m-%d')}"
        )
        output.append("")
        output.append(f"{Fore.YELLOW}Total commits:{Style.RESET_ALL} {stats.total_commits}")
        output.append(f"{Fore.YELLOW}Total files changed:{Style.RESET_ALL} {stats.total_files_changed}")
        output.append(f"{Fore.GREEN}Total lines added:{Style.RESET_ALL} {stats.total_lines_added}")
        output.append(f"{Fore.RED}Total lines deleted:{Style.RESET_ALL} {stats.total_lines_deleted}")
        
        net_change = stats.total_lines_added - stats.total_lines_deleted
        net_color = Fore.GREEN if net_change >= 0 else Fore.RED
        output.append(f"{Fore.YELLOW}Net change:{Style.RESET_ALL} {net_color}{net_change}{Style.RESET_ALL}")

        if stats.authors:
            output.append("")
            output.append(f"{Fore.MAGENTA}Contributors:{Style.RESET_ALL}")
            for author, count in sorted(
                stats.authors.items(), key=lambda x: x[1], reverse=True
            ):
                output.append(f"  {author}: {count} commits")

        return "\n".join(output)



"""Extended output formatter."""

import colorama
from colorama import Fore, Style

from ..core.models import CommitStats, RangeStats
from .standard import StandardFormatter

# Initialize colorama for cross-platform color support
colorama.init()

class ExtendedFormatter(StandardFormatter):
    """Extended formatter with additional details."""

    def format_commit_stats(self, stats: CommitStats) -> str:
        """Format commit statistics with extended details."""
        output = super().format_commit_stats(stats)

        if stats.files:
            # Add file type analysis
            file_types = {}
            for file_stat in stats.files:
                ext = (
                    file_stat.path.split(".")[-1]
                    if "." in file_stat.path
                    else "no-ext"
                )
                if ext not in file_types:
                    file_types[ext] = {
                        "count": 0,
                        "added": 0,
                        "deleted": 0,
                    }
                file_types[ext]["count"] += 1
                file_types[ext]["added"] += file_stat.lines_added
                file_types[ext]["deleted"] += file_stat.lines_deleted

            output += f"\n\n{Fore.MAGENTA}File type breakdown:{Style.RESET_ALL}"
            for ext, data in sorted(file_types.items()):
                output += (
                    f"\n  .{Fore.CYAN}{ext}{Style.RESET_ALL}: "
                    f"{data['count']} files, "
                    f"{Fore.GREEN}+{data['added']}{Style.RESET_ALL} "
                    f"{Fore.RED}-{data['deleted']}{Style.RESET_ALL}"
                )

        return output

    def format_range_stats(self, stats: RangeStats) -> str:
        """Format range statistics with extended details."""
        output = super().format_range_stats(stats)

        if stats.commits:
            # Add daily activity
            daily_activity = {}
            for commit in stats.commits:
                day = commit.date.strftime("%Y-%m-%d")
                if day not in daily_activity:
                    daily_activity[day] = 0
                daily_activity[day] += 1

            output += f"\n\n{Fore.MAGENTA}Daily activity:{Style.RESET_ALL}"
            for day, count in sorted(daily_activity.items()):
                output += f"\n  {Fore.CYAN}{day}{Style.RESET_ALL}: {count} commits"

        return output



"""Data models for Beacon delivery analytics."""

from dataclasses import dataclass
from datetime import datetime
from typing import Dict, List


@dataclass
class FileStats:
    """Statistics for a single file."""

    path: str
    lines_added: int
    lines_deleted: int
    lines_changed: int


@dataclass
class CommitStats:
    """Statistics for a single commit."""

    hash: str
    author: str
    date: datetime
    message: str
    files_changed: int
    lines_added: int
    lines_deleted: int
    files: List[FileStats]


@dataclass
class RangeStats:
    """Statistics for a range of commits."""

    start_date: datetime
    end_date: datetime
    total_commits: int
    total_files_changed: int
    total_lines_added: int
    total_lines_deleted: int
    commits: List[CommitStats]
    authors: Dict[str, int]



"""Beacon - Your delivery compass for empowered product builders."""

__version__ = "0.1.0"
__author__ = "Your Name"
__email__ = "your.email@example.com"

from .core.analyzer import GitAnalyzer
from .core.models import CommitStats, RangeStats

__all__ = ["GitAnalyzer", "CommitStats", "RangeStats"]



#!/usr/bin/env python
"""Command-line interface for Beacon."""

import argparse
import sys

from .core.analyzer import GitAnalyzer
from .formatters.extended import ExtendedFormatter
from .formatters.json_format import JSONFormatter
from .formatters.standard import StandardFormatter


def main() -> None:
    """Main CLI entry point."""
    parser = argparse.ArgumentParser(
        description=(
            "Beacon - Your delivery compass for empowered product builders"
        )
    )
    parser.add_argument(
        "commit",
        nargs="?",
        default="HEAD",
        help="Commit hash to analyze (default: HEAD)",
    )
    parser.add_argument(
        "-f",
        "--format",
        choices=["standard", "extended", "json"],
        default="standard",
        help="Output format (default: standard)",
    )
    parser.add_argument(
        "-r", "--range", action="store_true", help="Analyze range of commits"
    )
    parser.add_argument(
        "--since",
        default="1 week ago",
        help="Start date for range analysis (default: 1 week ago)",
    )
    parser.add_argument(
        "--until",
        default="HEAD",
        help="End date for range analysis (default: HEAD)",
    )
    parser.add_argument(
        "--repo",
        default=".",
        help="Repository path (default: current directory)",
    )

    args = parser.parse_args()

    try:
        analyzer = GitAnalyzer(args.repo)

        if args.range:
            stats = analyzer.get_range_analytics(args.since, args.until)
            if args.format == "json":
                formatter = JSONFormatter()
            else:
                formatter = StandardFormatter()
            print(formatter.format_range_stats(stats))
        else:
            stats = analyzer.get_commit_stats(args.commit)

            if args.format == "standard":
                formatter = StandardFormatter()
            elif args.format == "extended":
                formatter = ExtendedFormatter()
            else:
                formatter = JSONFormatter()

            print(formatter.format_commit_stats(stats))

    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()
